#include <iostream>
#include <vector>
#include <string>
#include <unistd.h>
#include <climits>
#include "defs.h"

namespace sys {
// NOTE: seems to be Linux-specific
static inline size_t getnumcores() {
    return sysconf(_SC_NPROCESSORS_ONLN);
}
#if 0
static inline bool threadaffinityisset(int &nthreads) {
    cpu_set_t mask;
    if (sched_getaffinity(0, sizeof(cpu_set_t), &mask) == -1) {
        perror("sched_getaffinity");
        throw std::runtime_error("sched_getaffinity failure");
    }
    int NC = getnumcores();
    int nset = 0;
    for (int i = 0; i < NC; i++) {
        nset += (CPU_ISSET(i, &mask) ? 1 : 0);
    }
    nthreads = nset;
    // We assume OK: exact one-to-one affinity or hyperthreading/SMT affinity
    // for 2, 3 or 4 threads
    return nthreads > 0 && nthreads < 5 && nthreads != NC;
}
#endif

static inline int getthreadaffinity(std::vector<int> &cores) {
    cpu_set_t mask;
    if (sched_getaffinity(0, sizeof(cpu_set_t), &mask) == -1) {
        perror("sched_getaffinity");
        throw std::runtime_error("sched_getaffinity failure");
    }
    int core = -1;
    for (size_t i = 0; i < sizeof(mask) * 8; i++) {
        if (CPU_ISSET(i, &mask)) {
            core = (int)i;
        cores.push_back(i);
        }
    }
    if (core == -1) {
        throw std::runtime_error("getthreadaffinity: logic error: cannot find an assigned core");
    }
    if (core >= (int)getnumcores()) {
        throw std::runtime_error("getthreadaffinity: logic error: wrong id of assigned core");
    }
    core = sched_getcpu();
    return core;
}
}

static inline std::string vinttostr(const std::vector<int> &cores) {
    std::string result;
    for (auto c : cores) {
        result += std::string(",") + std::to_string(c);
    }
    return std::string("{") + result.erase(0, 1) + "}";
}

void core_affinity(const std::vector<int> &params, std::vector<std::vector<double>> &output) {
    (void)output;
    int rank = params.front();
    std::vector<int> cores;
    int core = sys::getthreadaffinity(cores);
    print(rank, std::string("core affinity: ") + std::to_string(core) + "/" + vinttostr(cores));
}
