#include <vector>
#include "defs.h"
#include "timer.h"
#include "alloc.h"


void hip_memcopy(const std::vector<int> &params, std::vector<std::vector<double>> &output) {
    int rank = params[0];
    int distance = params[1];
    int ngpus = params[2];

    if (rank % distance) 
        return;

    real *d = nullptr, *h = nullptr;
    size_t N = 1<<26;
    int niters = 10, nwarmup = 3;
    h = (real *)sys::cpu_alloc(N * sizeof(real));
    d = (real *)sys::gpu_alloc(N * sizeof(real));
    print(rank, "GPU-CPU (Device-Host) memcopy benchmark:");
    if (distance == 1) {
        print(rank, "Using each GPU:");
    } else {
        print(rank, std::string("Using every ") + std::to_string(distance) + " GPU (" + std::to_string(ngpus/distance) + " GPUs per node)");
    }

    std::vector<double> result;

    {
        timer t(rank2str(rank) + "H2D", sizeof(real)*N, niters, result);
        for (int i = 0; i < niters + nwarmup; i++) {
            if (i == nwarmup)
                t.start();
            HIP_CALL(hipMemcpy(h, d, sizeof(real)*N, hipMemcpyHostToDevice));
        }
    }
    {
        timer t(rank2str(rank) + "D2H", sizeof(real)*N, niters, result);
        for (int i = 0; i < niters + nwarmup; i++) {
            if (i == nwarmup)
                t.start();
            HIP_CALL(hipMemcpy(d, h, sizeof(real)*N, hipMemcpyDeviceToHost));
        }
    }
    sys::gpu_free(d);
    sys::cpu_free(h);
    output.push_back(result);
}

